#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pgm.h"
#include "evaluate.h"

int num_el=14;

main(int argc, char *argv[])
{
  FILE *intensity, *orientation, *out;
  int i, j, count_i, count_j;
  double **elAvg, contrastRatio, total_avg;
  int target_x, target_y, index;
  int boundry;
  char im_name[50];
  int max, width, height;
  unsigned char *image;

  int passed, prevIndex;
  double prevRatio;

  intensity = fopen("inten.data", "r");
  out = fopen("regressionResults", "w");

  elAvg = (double **) malloc(num_el*sizeof(double *));
  for(i=0; i<num_el; i++)
    elAvg[i] = (double *) malloc(num_el*sizeof(double));

  boundry=ELSIZE;
  passed = 1;
  prevRatio = 0;
  prevIndex = 0;

  while(!feof(intensity))
  {
    fscanf(intensity, "%s %d %d %d\n", im_name, &target_x, &target_y, &index);

    max = read_pgm(im_name, &image, &width, &height);

    count_i=0;
    for(i=boundry; i<IMSIZE-boundry; i+=ELSIZE)
    {
      count_j=0;
      for(j=boundry; j<IMSIZE-boundry; j+=ELSIZE)
	elAvg[count_i][count_j++] = avgElement(image, i, j, width);
      count_i++;
    }

    free(image);

    contrastRatio = computeRatio(elAvg, target_x, target_y, &total_avg);

    if ( ((index) && (index<=130) && (contrastRatio>prevRatio)) ||
         ((index>130)  && (contrastRatio<prevRatio)) )
    {
      fprintf(out, "intensity regression failed:\n");
      fprintf(out, "\t%d\t%g\n", prevIndex, prevRatio);
      fprintf(out, "\t%d\t%g\n", index, contrastRatio);
      passed = 0;
    }
    fprintf(out, "%d\t%g\t%g\n", index, prevRatio, contrastRatio);
    prevRatio = contrastRatio;
    prevIndex = index;

   /* 
    fprintf(out, "%d\t%g\t%g\t%g\t%d\n", index, contrastRatio,
		elAvg[target_y-1][target_x-1], total_avg, num_el);

   if(index ==130)
   {
    for(i=0; i<num_el; i++)
    {
      for (j=0; j<num_el; j++)
	printf("%g ", elAvg[i][j]);
      printf("\n");
    }
    printf("\n\n");
   } 
   */

    for(i=0; i<num_el; i++) 
      for(j=0; j<num_el; j++)
        elAvg[i][j]=0;

  }
  if (passed) printf("Intensity test successful\n");


  orientation = fopen("orien.data", "r");
  passed=1;
  prevRatio = 0;
  prevIndex = 0;

  while(!feof(orientation))
  {
    fscanf(orientation, "%s %d %d %d\n", im_name, &target_x, &target_y, &index);

    max = read_pgm(im_name, &image, &width, &height);

    count_i=0;
    for(i=boundry; i<IMSIZE-boundry; i+=ELSIZE)
    {
      count_j=0;
      for(j=boundry; j<IMSIZE-boundry; j+=ELSIZE)
	elAvg[count_i][count_j++] = avgElement(image, i, j, width);
      count_i++;
    }

    free(image);

    contrastRatio = computeRatio(elAvg, target_x, target_y, &total_avg);

    if ( ((index<=90) && (contrastRatio<prevRatio)) ||
         ((index>90)  && (contrastRatio>prevRatio)) )
    {
      fprintf(out, "orientation regression failed:\n");
      fprintf(out, "\t%d\t%g\n", prevIndex, prevRatio);
      fprintf(out, "\t%d\t%g\n", index, contrastRatio);
      passed = 0;
    }
    fprintf(out, "%d\t%g\t%g\n", index, prevRatio, contrastRatio);
    prevRatio = contrastRatio;
    prevIndex = index;
  }

  if (passed) printf("Orientation test successful\n");

  for(i=0; i<num_el; i++)
    free(elAvg[i]);
  free(elAvg);

  fclose(intensity);
  fclose(orientation);
  fclose(out);
}

double avgElement(unsigned char *image, int x, int y, int width)
{
  int i, j;
  double total=0.0;

  for(i=0; i<ELSIZE; i++)
    for(j=0; j<ELSIZE; j++)
    {
/*      printf("row: %d, col: %d, value:%d\n", i+x, j+y,
		image[(i+x)*width+(j+y)]);                    */
      total+=image[(i+x)*width+j+y];
    }
  return total/(ELSIZE*ELSIZE);
}

double computeRatio(double **avg, int x, int y, double *total_avg)
{
  int i, j;
  double total=0.0;
  int count=0;

  for(i=2; i<num_el-2; i++)
  {
    for(j=2; j<num_el-2; j++)
      {
	total+=avg[i][j];
/*	printf("i=%d j=%d, avg=%g\n", i, j, avg[i][j]);*/
      }
  }

/*
  printf("total=%g ",total); 
*/
  total-=avg[y][x];

  *total_avg=total/((num_el-4)*(num_el-4)-1);
/*
  printf("total dist=%g target=%g total_avg=%g\n",total, avg[y][x],
		*total_avg); 
*/

  return avg[y][x]/(*total_avg);
}
